#!C:\ci\glue-core_1525089821216\_h_env\pythonw.exe
# EASY-INSTALL-ENTRY-SCRIPT: 'glue-core==0.12.5','gui_scripts','glue'
__requires__ = 'glue-core==0.12.5'
import re
import sys
from pkg_resources import load_entry_point

if __name__ == '__main__':
    sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0])
    sys.exit(
        load_entry_point('glue-core==0.12.5', 'gui_scripts', 'glue')()
    )
